from __future__ import annotations

ALPACA_PAPER_URL = "https://paper-api.alpaca.markets"
ALPACA_LIVE_URL = "https://api.alpaca.markets"

ACCOUNT_PATH = "/v2/account"

# positions
POSITIONS_PATH = "/v2/positions"
POSITION_PATH = "/v2/positions/{symbol_or_asset_id}"

# assets
ASSETS_PATH = "/v2/assets"
ASSET_PATH = "/v2/assets/{symbol_or_asset_id}"

# clock
CLOCK_PATH = "/v2/clock"

# orders
ORDERS_PATH = "/v2/orders"
ORDER_PATH = "/v2/orders/{order_id}"

# portfolio history
PORTFOLIO_HISTORY_PATH = f"{ACCOUNT_PATH}/portfolio/history"
